"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkGenerator = exports.containsJSONContent = exports.matchHostname = exports.addFetchInterceptor = void 0;
const logger_1 = require("../../lib/logger");
const types_1 = require("../../types");
const normalize_url_1 = require("./normalize-url");
let origFetch;
function addFetchInterceptor(onRequest, onResponse) {
    logger_1.logger.debug('Adding fetch interceptor');
    origFetch = window.fetch;
    window.fetch = async (...args) => {
        try {
            onRequest(args);
        }
        catch (err) {
            console.log('Error handling request: ', err);
        }
        const response = await origFetch(...args);
        try {
            onResponse(response.clone());
        }
        catch (err) {
            console.log('Error handling response: ', err);
        }
        return response;
    };
}
exports.addFetchInterceptor = addFetchInterceptor;
const matchHostname = (url) => {
    const rIsAbs = new RegExp('^(?:[a-z+]+:)?//', 'i');
    if (!rIsAbs.test(url)) {
        // Relative URL will go to this host
        return true;
    }
    const clean = new URL(url).hostname.replace('www.', '');
    const current = window.location.hostname.replace('www.', '');
    return clean.includes(current) || current.includes(clean);
};
exports.matchHostname = matchHostname;
const normalizeHeaders = (headers) => {
    return headers instanceof Headers ? headers : new Headers(headers);
};
const containsJSONContent = (headers) => {
    if (!headers) {
        return false;
    }
    const normalizedHeaders = normalizeHeaders(headers);
    return normalizedHeaders.get('content-type') === 'application/json';
};
exports.containsJSONContent = containsJSONContent;
class NetworkGenerator {
    constructor() {
        this.id = 'network';
    }
    register(emitter) {
        const handleRequest = ([url, rq]) => {
            if (!rq || !rq.body) {
                return;
            }
            const sUrl = url?.toString();
            if (!url || !(0, exports.matchHostname)(sUrl)) {
                return;
            }
            if (!(0, exports.containsJSONContent)(rq.headers)) {
                return;
            }
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'request',
                url: (0, normalize_url_1.normalizeUrl)(sUrl),
                method: rq.method || '',
                data: JSON.parse(rq.body.toString()),
            }));
        };
        const handleResponse = async (rs) => {
            if (!(0, exports.containsJSONContent)(rs.headers)) {
                return;
            }
            const url = rs.url;
            if (!url || !(0, exports.matchHostname)(url)) {
                return;
            }
            const data = await rs.json();
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'response',
                url: url,
                data: data,
            }));
        };
        addFetchInterceptor(handleRequest, handleResponse);
        return () => {
            logger_1.logger.debug('Removing fetch interceptor');
            window.fetch = origFetch;
        };
    }
}
exports.NetworkGenerator = NetworkGenerator;
//# sourceMappingURL=network-gen.js.map