"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignalGlobalSettings = void 0;
const debug_mode_1 = require("../debug-mode");
const logger_1 = require("../../lib/logger");
/**
 * Global settings for the application
 * In the future this pattern may allows us to add settings which can be dynamically set by the user.
 * Currently, this is just a way to pass settings to the different parts of the application.
 */
class SignalGlobalSettings {
    constructor(settings) {
        this.redaction = new SignalRedactionSettings();
        if (settings.maxBufferSize && settings.signalStorage) {
            throw new Error('maxBufferSize and signalStorage cannot be defined at the same time');
        }
        this.redaction = new SignalRedactionSettings(settings.disableSignalRedaction);
        this.signalBuffer = {
            signalStorage: settings.signalStorage,
            maxBufferSize: settings.maxBufferSize,
        };
        this.ingestClient = {
            apiHost: settings.apiHost,
            flushAt: settings.flushAt,
            flushInterval: settings.flushInterval,
            shouldDisableSignalRedaction: this.redaction.getDisableSignalRedaction,
        };
        this.sandbox = {
            functionHost: settings.functionHost,
            processSignal: settings.processSignal,
            edgeFnDownloadURL: undefined,
        };
    }
    update({ edgeFnDownloadURL }) {
        edgeFnDownloadURL && (this.sandbox.edgeFnDownloadURL = edgeFnDownloadURL);
    }
}
exports.SignalGlobalSettings = SignalGlobalSettings;
class SignalRedactionSettings {
    constructor(initialValue) {
        if (typeof initialValue === 'boolean') {
            this.setDisableSignalRedaction(initialValue);
        }
        // setting ?segment_signals_debug=true will disable redaction, and set a key in local storage
        // this setting will persist across page loads (even if there is no query string)
        // in order to clear the setting, user must set ?segment_signals_debug=false
        const debugModeInQs = (0, debug_mode_1.parseDebugModeQueryString)();
        logger_1.logger.debug('debugMode is set to true via query string');
        if (typeof debugModeInQs === 'boolean') {
            this.setDisableSignalRedaction(debugModeInQs);
        }
    }
    setDisableSignalRedaction(shouldDisable) {
        try {
            if (shouldDisable) {
                window.sessionStorage.setItem(SignalRedactionSettings.redactionKey, 'true');
            }
            else {
                logger_1.logger.debug('Removing redaction key from storage');
                window.sessionStorage.removeItem(SignalRedactionSettings.redactionKey);
            }
        }
        catch (e) {
            logger_1.logger.debug('Storage error', e);
        }
    }
    getDisableSignalRedaction() {
        try {
            const isDisabled = Boolean(window.sessionStorage.getItem(SignalRedactionSettings.redactionKey));
            if (isDisabled) {
                logger_1.logger.debug(`${SignalRedactionSettings.redactionKey}=true (app. storage)`);
                return true;
            }
        }
        catch (e) {
            logger_1.logger.debug('Storage error', e);
        }
        return false;
    }
}
SignalRedactionSettings.redactionKey = 'segment_signals_debug_redaction_disabled';
//# sourceMappingURL=settings.js.map