import { logger } from '../../lib/logger';
import { createNetworkSignal } from '../../types';
import { normalizeUrl } from './normalize-url';
let origFetch;
export function addFetchInterceptor(onRequest, onResponse) {
    logger.debug('Adding fetch interceptor');
    origFetch = window.fetch;
    window.fetch = async (...args) => {
        try {
            onRequest(args);
        }
        catch (err) {
            console.log('Error handling request: ', err);
        }
        const response = await origFetch(...args);
        try {
            onResponse(response.clone());
        }
        catch (err) {
            console.log('Error handling response: ', err);
        }
        return response;
    };
}
export const matchHostname = (url) => {
    const rIsAbs = new RegExp('^(?:[a-z+]+:)?//', 'i');
    if (!rIsAbs.test(url)) {
        // Relative URL will go to this host
        return true;
    }
    const clean = new URL(url).hostname.replace('www.', '');
    const current = window.location.hostname.replace('www.', '');
    return clean.includes(current) || current.includes(clean);
};
const normalizeHeaders = (headers) => {
    return headers instanceof Headers ? headers : new Headers(headers);
};
export const containsJSONContent = (headers) => {
    if (!headers) {
        return false;
    }
    const normalizedHeaders = normalizeHeaders(headers);
    return normalizedHeaders.get('content-type') === 'application/json';
};
export class NetworkGenerator {
    constructor() {
        this.id = 'network';
    }
    register(emitter) {
        const handleRequest = ([url, rq]) => {
            if (!rq || !rq.body) {
                return;
            }
            const sUrl = url?.toString();
            if (!url || !matchHostname(sUrl)) {
                return;
            }
            if (!containsJSONContent(rq.headers)) {
                return;
            }
            emitter.emit(createNetworkSignal({
                action: 'request',
                url: normalizeUrl(sUrl),
                method: rq.method || '',
                data: JSON.parse(rq.body.toString()),
            }));
        };
        const handleResponse = async (rs) => {
            if (!containsJSONContent(rs.headers)) {
                return;
            }
            const url = rs.url;
            if (!url || !matchHostname(url)) {
                return;
            }
            const data = await rs.json();
            emitter.emit(createNetworkSignal({
                action: 'response',
                url: url,
                data: data,
            }));
        };
        addFetchInterceptor(handleRequest, handleResponse);
        return () => {
            logger.debug('Removing fetch interceptor');
            window.fetch = origFetch;
        };
    }
}
//# sourceMappingURL=network-gen.js.map